@@ -1,26 +1,16 @@
-/// A simple example demonstrating how to handle user input. This is
-/// a bit out of the scope of the library as it does not provide any
-/// input handling out of the box. However, it may helps some to get
-/// started.
-///
-/// This is a very simple example:
-///   * A input box always focused. Every character you type is registered
-///   here
-///   * Pressing Backspace erases a character
-///   * Pressing Enter pushes the current input in the history of previous
-///   messages
 //use ruma_identifiers::{RoomId, UserId};
 //use serde::{Serialize, Deserialize};
-use scanpw::scanpw;
-use serde::{Serialize, Deserialize};
-use crackle_lib::{RoomId, UserId};
+use crackle_lib::{RoomId, RoomName, UserId};
 use crossbeam_channel::{unbounded, TryRecvError};
 use crossterm::{
     event::{self, DisableMouseCapture, EnableMouseCapture, Event, KeyCode},
     execute,
     terminal::{disable_raw_mode, enable_raw_mode, EnterAlternateScreen, LeaveAlternateScreen},
 };
-//use std::convert::TryFrom;
+use scanpw::scanpw;
+use serde::{Deserialize, Serialize};
+use std::collections::BTreeMap;
+use std::convert::TryFrom;
 //use ruma_client_api::sync;
 use std::{error::Error, io};
 use tui::{
@@ -36,10 +26,10 @@ use tui::{
 use unicode_width::UnicodeWidthStr;
 //use crossbeam_utils::atomic::AtomicCell;
 
-//static message_box_height: AtomicCell<u16> = AtomicCell::new(3); 
+//static message_box_height: AtomicCell<u16> = AtomicCell::new(3);
 #[derive(Serialize, Deserialize, Debug, Clone)]
 struct CrackleConfig {
-    matrix_url: String, 
+    matrix_url: String,
     auth_token: String,
     username: String,
     room: String,
@@ -54,6 +44,7 @@ enum Focus {
     //Login,
     //Register,
     //RoomSearch,
+    RoomSelect,
 }
 
 /// App holds the state of the application
@@ -63,12 +54,15 @@ struct App {
     /// Current input mode
     focus: Focus,
     /// History of recorded messages
-    messages: Vec<String>,
+    messages: BTreeMap<RoomId, Vec<String>>,
     host: String,
     server_name: String,
     user_name: String,
     password: Option<String>,
+    selected_room: Option<RoomId>,
     room: String,
+    rooms: BTreeMap<RoomId, Box<RoomName>>,
+    current_room: Option<RoomId>,
     //using_config: bool,
     auth_token: Option<String>,
     //message_box_height: u16,
@@ -86,16 +80,27 @@ impl Default for App {
 }
 */
 
-
 impl App {
-    pub async fn new(home_server_url: impl Into<String>, user_name: String,  password: Option<String>, room: String, auth_token: Option<String>) -> Self {
+    pub async fn new(
+        home_server_url: impl Into<String>,
+        user_name: String,
+        password: Option<String>,
+        room: String,
+        auth_token: Option<String>,
+    ) -> Self {
         let mut hs_url = home_server_url.into();
         //let mut host: String;
         if !hs_url.starts_with("http://") && !hs_url.starts_with("https://") {
             hs_url = format!("https://{}", hs_url);
         }
         let hostparts = hs_url.split_once("://").unwrap();
-        let server_name = hostparts.1.clone().split(":").map(|z| z.to_string()).collect::<Vec<String>>()[0].clone();
+        let server_name = hostparts
+            .1
+            .clone()
+            .split(":")
+            .map(|z| z.to_string())
+            .collect::<Vec<String>>()[0]
+            .clone();
         let username = match UserId::try_from(user_name.clone()) {
             Ok(user_id) => user_id.localpart().to_string(),
             Err(_) => user_name.clone(),
@@ -103,13 +108,16 @@ impl App {
         Self {
             input: String::new(),
             focus: Focus::Watching,
-            messages: Vec::new(),
+            messages: BTreeMap::new(),
             host: hs_url,
             server_name: server_name,
             user_name: username,
             password: password,
+            selected_room: None,
+            current_room: None,
+            rooms: BTreeMap::new(),
+            auth_token: auth_token,
             room: room,
-            auth_token: auth_token, 
             //message_box_height: 3,
             //client: ruma_client::Client::builder().homeserver_url(home_server.into()).build::<ruma_client::http_client::Reqwest>().await.unwrap(),
         }
@@ -125,7 +133,8 @@ async fn main() -> Result<(), Box<dyn Error>> {
     let auth_token: Option<String>;
     let mut room: String;
     if std::path::Path::new("./config.yaml").exists() {
-        let config: CrackleConfig = serde_yaml::from_str(&std::fs::read_to_string("./config.yaml").unwrap()).unwrap();
+        let config: CrackleConfig =
+            serde_yaml::from_str(&std::fs::read_to_string("./config.yaml").unwrap()).unwrap();
         host = config.matrix_url;
         username = config.username;
         auth_token = Some(config.auth_token);
@@ -152,7 +161,14 @@ async fn main() -> Result<(), Box<dyn Error>> {
     let mut terminal = Terminal::new(backend)?;
 
     // create app and run it
-    let app = App::new(host.trim().to_string(), username.trim().to_string(), password, room.trim().to_string(), auth_token).await;
+    let app = App::new(
+        host.trim().to_string(),
+        username.trim().to_string(),
+        password,
+        room.trim().to_string(),
+        auth_token,
+    )
+    .await;
     let res = run_app(&mut terminal, app).await;
 
     // restore terminal
@@ -188,12 +204,12 @@ enum SyncMessage {
     Quit,
     //Response(sync::sync_events::v3::Response),
     SendMessage(RoomId, String, String),
+    JoinedRooms(BTreeMap<RoomId, Box<RoomName>>),
 }
 #[allow(unused_assignments)]
 async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Result<()> {
     /*
     let mut client = ruma_client::Client::builder().homeserver_url("https://sonicrules.fun:8448/".parse().unwrap()).build::<ruma_client::http_client::Reqwest>().await.unwrap();//.access_token(Some(std::fs::read_to_string("auth_token.txt").unwrap().trim().to_string())).build();
-    
     let mut login_result = client.log_in("@crackle_client:sonicrules.fun", "50nicru1e5", None, None).await;
     while login_result.is_err() {
         login_result = client.log_in("@crackle_client:sonicrules.fun", "50nicru1e5", None, None).await;
@@ -208,13 +224,16 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
     */
     //let distrotube_room_id = client.send_request(ruma_client_api::alias::get_alias::v3::Request::new(ruma_identifiers::room_alias_id!("#distrotube:matrix.org"))).await.unwrap().room_id;
     //println!("'{}'", app.server_name.clone());
-    let mut client = crackle_lib::Client::new(app.host.clone(), app.user_name.clone(), app.server_name.clone());
+    let mut client = crackle_lib::Client::new(
+        app.host.clone(),
+        app.user_name.clone(),
+        app.server_name.clone(),
+    );
     if let Some(pass) = app.password.clone() {
         client.login(pass);
     } else {
         client.set_auth_token(app.auth_token.as_ref().unwrap().clone());
     }
-    
     let me = client.get_me();
     let (tx1, rx1) = unbounded::<SyncMessage>();
     let tx1_clone = tx1.clone();
@@ -225,6 +244,10 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
     //let tx_clone = tx.clone();
     let networking = tokio::task::spawn(async move {
         let mut sync_results = client.sync().unwrap();
+        //let mut known_room_list: Vec<RoomId> = Vec::new();
+        client.extract_data(sync_results.clone());
+        //let mut f = std::fs::File::create("sync.log").unwrap();
+        //writeln!(f, "{:#?}", sync_results);
         //client.join_with_alias(app.room);
         //let mut sync_results = sync(client.clone(), None, true).await;
         //let mut since = sync_results.next_batch;
@@ -236,26 +259,110 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                 Ok(SyncMessage::SendMessage(room_id, _user_id, message)) => {
                     client.send_message(room_id.try_into().unwrap(), message);
                 }
-                Err(TryRecvError::Empty) => {},
+                Ok(_) => {}
+                Err(TryRecvError::Empty) => {}
                 Err(_) => break,
             }
             sync_results = client.sync().unwrap();
-            let messages = client.extract_messages(sync_results.clone());//, client.clone().me);
+            let messages = client.extract_data(sync_results.clone()); //, client.clone().me);
+            if client.room_list_updated() {
+                tx1.send(SyncMessage::JoinedRooms(client.data.room_to_name.clone()))
+                    .unwrap();
+            }
             for message in messages {
                 let room_id = message.room_id;
                 let user_id = message.sender.to_string();
                 let msg = message.message.to_string();
                 //if room_id.clone() == distrotube_room_id {
-                tx1.send(SyncMessage::SendMessage(room_id, user_id, msg)).unwrap();
+                tx1.send(SyncMessage::SendMessage(room_id, user_id, msg))
+                    .unwrap();
                 //}
             }
-        }        
+        }
     });
     loop {
-        terminal.draw(|f| ui(f, &app))?;
+        terminal.draw(|f| ui(f, &mut app))?;
         if event::poll(std::time::Duration::from_secs(0)).unwrap() {
             if let Event::Key(key) = event::read()? {
                 match app.focus {
+                    Focus::RoomSelect => {
+                        let num_rooms = app.rooms.len();
+                        if num_rooms > 1 {
+                            /*
+                            let mut prev: RoomId;
+                            let mut next: RoomId;
+                            let last = app.rooms.iter().last().unwrap().0;
+                            let first: RoomId;
+                            for (index_num, (room_id, _room_name)) in app.rooms.iter().enumerate() {
+                                if index_num == 0 {
+                                    first = room_id;
+                                }
+                            }
+                            */
+                            match key.code {
+                                KeyCode::Up => {
+                                    let last = app.rooms.iter().last().unwrap().0;
+                                    let first = app.rooms.iter().nth(0).unwrap().0;
+                                    let mut prev = last.clone();
+                                    for (index_num, (room_id, _room_name)) in
+                                        app.rooms.iter().enumerate()
+                                    {
+                                        if index_num == 0 {
+                                            if first.clone() == app.selected_room.clone().unwrap() {
+                                                app.selected_room = Some(last.clone());
+                                                break;
+                                            }
+                                            prev = room_id.clone();
+                                        } else {
+                                            if room_id.clone() == app.selected_room.clone().unwrap()
+                                            {
+                                                app.selected_room = Some(prev);
+                                                break;
+                                            }
+                                            prev = room_id.clone();
+                                        } //else {
+                                          //    app.selected_room = Some(last.clone());
+                                          //}
+                                    }
+                                }
+                                KeyCode::Down => {
+                                    let last = app.rooms.iter().last().unwrap().0;
+                                    let first = app.rooms.iter().nth(0).unwrap().0;
+                                    let mut use_next = false;
+                                    for (room_id, _room_name) in app.rooms.iter() {
+                                        if use_next {
+                                            app.selected_room = Some(room_id.clone());
+                                            break;
+                                        }
+                                        /*else if index_num == 0 {
+                                            //first = room_id.clone();
+                                            if first.clone() == app.selected_room.clone().unwrap() {
+                                                use_next = true;
+                                            }
+                                        }*/
+                                        else if room_id != last {
+                                            if room_id.clone() == app.selected_room.clone().unwrap()
+                                            {
+                                                use_next = true;
+                                            }
+                                        } else {
+                                            app.selected_room = Some(first.clone());
+                                        }
+                                    }
+                                }
+                                KeyCode::Enter => {
+                                    app.current_room = Some(app.selected_room.clone().unwrap());
+                                    app.selected_room = None;
+                                    app.focus = Focus::Watching;
+                                }
+                                KeyCode::Esc => {
+                                    app.selected_room = None;
+                                    app.focus = Focus::Watching;
+                                }
+                                _ => {}
+                            }
+                        }
+                    }
                     Focus::Watching => match key.code {
                         KeyCode::Char('e') => {
                             app.focus = Focus::MessageInput;
@@ -266,6 +373,9 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                             networking.await.unwrap();
                             return Ok(());
                         }
+                        KeyCode::Char('r') => {
+                            app.focus = Focus::RoomSelect;
+                        }
                         _ => {
                             /*
                             if let Ok(msg) = rx1.try_recv() {
@@ -280,10 +390,13 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                         KeyCode::Enter => {
                             let message: String = app.input.drain(..).collect();
                             //app.messages.push(message.clone());
-                            let sync_message = SyncMessage::SendMessage(sm_room_id.clone(), format!("{}", me), message);
+                            let sync_message = SyncMessage::SendMessage(
+                                app.current_room.clone().unwrap(),
+                                format!("{}", me),
+                                message,
+                            );
                             tx1_clone.send(sync_message.clone()).unwrap();
                             tx2.send(sync_message.clone()).unwrap();
-                            
                         }
                         KeyCode::Char(c) => {
                             app.input.push(c);
@@ -309,17 +422,29 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
         } else {
             if let Ok(msg) = rx1.try_recv() {
                 if let SyncMessage::SendMessage(room_id, user_id, message) = msg {
-                    if app.messages.len() > 100 {
-                        app.messages.remove(0);
+                    let mut msgs = app.messages[&room_id].clone();
+                    if msgs.len() > 100 {
+                        msgs.remove(0);
+                    }
+                    msgs.push(format!("<{}> {}", user_id, message));
+                    app.messages.insert(room_id.clone(), msgs).unwrap();
+                } else if let SyncMessage::JoinedRooms(room_id_list) = msg {
+                    for (room_id, room_name) in room_id_list {
+                        if app.current_room.is_none() {
+                            app.current_room = Some(room_id.clone());
+                        }
+                        if !app.rooms.contains_key(&room_id.clone()) {
+                            app.rooms.insert(room_id.clone(), room_name.clone());
+                            app.messages.insert(room_id.clone(), Vec::new());
+                        }
                     }
-                    app.messages.push(format!("[{}] <{}> {}", room_id, user_id, message));
                 }
             }
         }
     }
 }
 
-fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
+fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
     let chunks = Layout::default()
         .direction(Direction::Vertical)
         //.margin(2)
@@ -333,6 +458,10 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
         )
         .split(f.size());
     //message_box_height.store(chunks[1].height);
+    let middle_chunk = Layout::default()
+        .direction(Direction::Horizontal)
+        .constraints([Constraint::Percentage(20), Constraint::Percentage(80)].as_ref())
+        .split(chunks[1]);
     let (msg, style) = match app.focus {
         Focus::Watching => (
             vec![
@@ -340,7 +469,9 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
                 Span::styled("q", Style::default().add_modifier(Modifier::BOLD)),
                 Span::raw(" to exit, "),
                 Span::styled("e", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to start editing."),
+                Span::raw(" to start editing, or "),
+                Span::styled("r", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to choose a room "),
             ],
             Style::default().add_modifier(Modifier::RAPID_BLINK),
         ),
@@ -354,6 +485,16 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
             ],
             Style::default(),
         ),
+        Focus::RoomSelect => (
+            vec![
+                Span::raw("Press "),
+                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to stop choosing a room with the Up and Down arrow keys, "),
+                Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to choose the selected room"),
+            ],
+            Style::default(),
+        ),
     };
     let mut text = Text::from(Spans::from(msg));
     text.patch_style(style);
@@ -364,6 +505,7 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
         .style(match app.focus {
             Focus::Watching => Style::default(),
             Focus::MessageInput => Style::default().fg(Color::Yellow),
+            Focus::RoomSelect => Style::default(),
         })
         .block(Block::default().borders(Borders::ALL).title("Input"));
     match app.focus {
@@ -380,111 +522,93 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
                 chunks[2].y + 1,
             )
         }
+        Focus::RoomSelect => {}
     }
-    let messages_len: i32 = app.messages.len().try_into().unwrap();
-    let height: i32 = chunks[1].height.try_into().unwrap(); //message_box_height.load().try_into().unwrap();
-    let mut cut_off = messages_len - height + 2;
-    if cut_off <= 0 {
-        cut_off = 0;
-    }
-    let messages: Vec<ListItem> = app
-        .messages[cut_off.try_into().unwrap()..]
+    let room_list: Vec<ListItem> = app
+        .rooms
         .iter()
-        .enumerate()
-        .map(|(_i, m)| {
-            let content = vec![Spans::from(Span::raw(format!("{}", m)))];
-            ListItem::new(content)
+        //.enumerate()
+        .map(|(room_id, room_name)| {
+            let content = vec![Spans::from(Span::raw(format!("{}", room_name)))];
+            //let content: Vec<Span>;
+            if app.current_room.is_none() {
+                app.current_room = Some(room_id.clone());
+            }
+            let current_room = app.current_room.clone().unwrap();
+            //if let Some(room) = current_room {
+            /*
+            if current_room == room_id.clone() {
+                content = vec![Span::from(Span::styled(
+                    format!("{}", room_name),
+                    Style::default().add_modifier(Modifier::BOLD),
+                ))];
+            } else {
+                content = vec![Span::from(Span::raw(format!("{}", room_name)))];
+            }
+            */
+            //}
+            let mut z = ListItem::new(content);
+            let mut style = Style::default();
+            if room_id.clone() == current_room {
+                style = style.add_modifier(Modifier::BOLD);
+                z = z.style(style);
+            }
+            match app.focus {
+                Focus::RoomSelect => {
+                    if app.selected_room.is_none() {
+                        app.selected_room = Some(current_room);
+                    }
+                    if app.selected_room.clone().unwrap() == room_id.clone() {
+                        z = z.style(style.add_modifier(Modifier::REVERSED));
+                    }
+                }
+                _ => {}
+            }
+            z
         })
         .collect();
-    let messages =
-        List::new(messages).block(Block::default().borders(Borders::ALL).title("Messages"));
-    f.render_widget(messages, chunks[1]);
-    f.render_widget(input, chunks[2]);
-    //println!("{}", chunks[1].height);
-}
-/*
-fn login_ui<B: Backend>(f: &mut Frame<B>, app: &App) {
-    let chunks = Layout::default()
-        .direction(Direction::Vertical)
-        //.margin(2)
-        .constraints(
-            [
-                Constraint::Length(1),
-                Constraint::Length(3),
-                Constraint::Length(3),
-                Constraint::Length(3),
-            ]
-            .as_ref(),
+    let room_list =
+        List::new(room_list).block(Block::default().borders(Borders::ALL).title("Room List"));
+    let mut msg_list: Vec<ListItem> = Vec::new();
+    if app.current_room.is_some() {
+        let ms = app.messages[&app.current_room.clone().unwrap()].join("\n");
+        //let width: i32 = middle_chunk[1].width.try_into().unwrap(); //message_box_height.load().try_into().unwrap();
+        let lines: Vec<String> = textwrap::wrap(
+            ms.as_str(),
+            textwrap::Options::new((middle_chunk[1].width - 2).into()),
         )
-        .split(f.size());
-    //message_box_height.store(chunks[1].height);
-    let (msg, style) = match app.focus {
-        Focus::Watching => (
-            vec![
-                Span::raw("Press "),
-                Span::styled("q", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to exit, "),
-                Span::styled("e", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to start editing."),
-            ],
-            Style::default().add_modifier(Modifier::RAPID_BLINK),
-        ),
-        Focus::MessageInput => (
-            vec![
-                Span::raw("Press "),
-                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to stop editing, "),
-                Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to record the message"),
-            ],
-            Style::default(),
-        ),
-    };
-    let mut text = Text::from(Spans::from(msg));
-    text.patch_style(style);
-    let help_message = Paragraph::new(text);
-    f.render_widget(help_message, chunks[0]);
-
-    let input = Paragraph::new(app.input.as_ref())
-        .style(match app.focus {
-            Focus::Watching => Style::default(),
-            Focus::MessageInput => Style::default().fg(Color::Yellow),
-        })
-        .block(Block::default().borders(Borders::ALL).title("Input"));
-    match app.focus {
-        Focus::Watching =>
-            // Hide the cursor. `Frame` does this by default, so we don't need to do anything here
-            {}
-
-        Focus::MessageInput => {
-            // Make the cursor visible and ask tui-rs to put it at the specified coordinates after rendering
-            f.set_cursor(
-                // Put cursor past the end of the input text
-                chunks[2].x + app.input.width() as u16 + 1,
-                // Move one line down, from the border to the input line
-                chunks[2].y + 1,
-            )
-        }
-    }
-    let messages_len: i32 = app.messages.len().try_into().unwrap();
-    let height: i32 = chunks[1].height.try_into().unwrap(); //message_box_height.load().try_into().unwrap();
-    let mut cut_off = messages_len - height + 2;
-    if cut_off <= 0 {
-        cut_off = 0;
-    }
-    let messages: Vec<ListItem> = app
-        .messages[cut_off.try_into().unwrap()..]
         .iter()
-        .enumerate()
-        .map(|(_i, m)| {
-            let content = vec![Spans::from(Span::raw(format!("{}", m)))];
-            ListItem::new(content)
-        })
+        .map(|x| x.to_string())
         .collect();
+        //.split("\n")
+        //.map(|x| x.to_string())
+        //.collect();
+        /*)
+        let messages_len: i32 = app.messages[&app.current_room.clone().unwrap()]
+            .len()
+            .try_into()
+            .unwrap();
+        */
+        let messages_len: i32 = lines.len().try_into().unwrap();
+        let height: i32 = middle_chunk[1].height.try_into().unwrap(); //message_box_height.load().try_into().unwrap();
+        let mut cut_off = messages_len - height + 2;
+        if cut_off <= 0 {
+            cut_off = 0;
+        }
+        //msg_list = app.messages[app.current_room.as_ref().unwrap()][cut_off.try_into().unwrap()..]
+        msg_list = lines[cut_off.try_into().unwrap()..]
+            .iter()
+            .enumerate()
+            .map(|(_i, m)| {
+                let content = vec![Spans::from(Span::raw(format!("{}", m)))];
+                ListItem::new(content)
+            })
+            .collect();
+    }
     let messages =
-        List::new(messages).block(Block::default().borders(Borders::ALL).title("Messages"));
-    f.render_widget(messages, chunks[1]);
+        List::new(msg_list).block(Block::default().borders(Borders::ALL).title("Messages"));
+    f.render_widget(messages, middle_chunk[1]);
     f.render_widget(input, chunks[2]);
+    f.render_widget(room_list, middle_chunk[0]);
     //println!("{}", chunks[1].height);
 }
-*/
\ No newline at end of file
